# python 2 backwards compatibility
from __future__ import print_function
from builtins import object, super
from future import standard_library
from six import string_types

# package imports
from .base import ModelBase
from .enum import *
from .tag import Tag


class Indicator(ModelBase):
    """
    Models an |Indicator_resource|.

    :ivar value: The indicator value; i.e. "www.evil.com"
    :ivar type: The type of indicator; i.e. "URL"
    :ivar priority_level: The priority level of the indicator
    :ivar correlation_count: The number of other indicators that are correlated with this indicator.
    :ivar whitelisted: Whether the indicator is whitelisted or not.
    :ivar weight: see |Indicator_resource| for details.
    :ivar reason: see |Indicator_resource| for details.
    :ivar first_seen: the first time this indicator was sighted
    :ivar last_seen: the last time this indicator was sighted
    :ivar sightings: the number of times this indicator has been sighted
    :ivar source: the source that the indicator was observed from
    :ivar notes: a string containing notes about the indicator
    :ivar tags: a list containing |Tag| objects associated with the indicator
    :ivar enclave_ids: a list of enclaves that the indicator is found in

    :cvar TYPES: A list of all valid indicator types.
    """

    TYPES = IndicatorType.values()

    def __init__(self,
                 value,
                 type=None,
                 priority_level=None,
                 correlation_count=None,
                 whitelisted=None,
                 weight=None,
                 reason=None,
                 first_seen=None,
                 last_seen=None,
                 sightings=None,
                 source=None,
                 notes=None,
                 tags=None,
                 enclave_ids=None):

        self.value = value
        self.type = type
        self.priority_level = priority_level
        self.correlation_count = correlation_count
        self.whitelisted = whitelisted
        self.weight = weight
        self.reason = reason

        # ioc management fields
        self.first_seen = first_seen
        self.last_seen = last_seen
        self.sightings = sightings
        self.source = source
        self.notes = notes
        self.tags = tags
        self.enclave_ids = enclave_ids

    @classmethod
    def from_dict(cls, indicator):
        """
        Create an indicator object from a dictionary.

        :param indicator: The dictionary.
        :return: The indicator object.
        """

        tags = indicator.get('tags')
        if tags is not None:
            tags = [Tag.from_dict(tag) for tag in tags]

        return Indicator(value=indicator.get('value'),
                         type=indicator.get('indicatorType'),
                         priority_level=indicator.get('priorityLevel'),
                         correlation_count=indicator.get('correlationCount'),
                         whitelisted=indicator.get('whitelisted'),
                         weight=indicator.get('weight'),
                         reason=indicator.get('reason'),
                         first_seen=indicator.get('firstSeen'),
                         last_seen=indicator.get('lastSeen'),
                         sightings=indicator.get('sightings'),
                         source=indicator.get('source'),
                         notes=indicator.get('notes'),
                         tags=tags,
                         enclave_ids=indicator.get('enclaveIds'))

    def to_dict(self, remove_nones=False):
        """
        Creates a dictionary representation of the indicator.

        :param remove_nones: Whether ``None`` values should be filtered out of the dictionary.  Defaults to ``False``.
        :return: A dictionary representation of the indicator.
        """

        if remove_nones:
            return super(Indicator, self).to_dict(remove_nones=True)

        tags = None
        if self.tags is not None:
            tags = [tag.to_dict(remove_nones=remove_nones) for tag in self.tags]

        return {
            'value': self.value,
            'indicatorType': self.type,
            'priorityLevel': self.priority_level,
            'correlationCount': self.correlation_count,
            'whitelisted': self.whitelisted,
            'weight': self.weight,
            'reason': self.reason,
            'firstSeen': self.first_seen,
            'lastSeen': self.last_seen,
            'source': self.source,
            'sightings': self.sightings,
            'notes': self.notes,
            'tags': tags,
            'enclaveIds': self.enclave_ids
        }



from tests.conftest import BASE_URL
from trustar import CursorPage

PRIORITY_EVENT_SCORE = (3, 2, 1)
NORMALIZED_INDICATOR_SCORE = (3, 2, 1)
STATUSES = ("UNRESOLVED", "CONFIRMED", "IGNORED")
URL_ENDPOINT = BASE_URL + "/triage"


def test_get_phishing_submissions_page(mocked_request, trustar):
    mocked_request.post(url=f"{URL_ENDPOINT}/submissions", json={"items": [{"submissionId": "1234"}],
                                                                 'responseMetadata': {'nextCursor': ''}})
    page = trustar.get_phishing_submissions_page(priority_event_score=PRIORITY_EVENT_SCORE,
                                                 status=STATUSES)
    assert isinstance(page, CursorPage)
    assert isinstance(page.items, list)
    assert isinstance(page.response_metadata, dict)
    expected = {'items': [{'submissionId': '1234'}], 'responseMetadata': {'nextCursor': ''}}
    assert expected == page.to_dict(remove_nones=True)


def test_get_phishing_indicators_page(mocked_request, trustar):
    mocked_request.post(url=f"{URL_ENDPOINT}/indicators", json={'items': [
        {'indicatorType': 'IP', 'value': '220.178.71.156', 'sourceKey': 'alienvault_otx'}],
        'responseMetadata': {'nextCursor': ''}})
    page = trustar.get_phishing_indicators_page(
        normalized_indicator_score=NORMALIZED_INDICATOR_SCORE,
        priority_event_score=PRIORITY_EVENT_SCORE,
        status=STATUSES)
    assert isinstance(page, CursorPage)
    assert isinstance(page.items, list)
    assert isinstance(page.response_metadata, dict)
    expected = {'items': [{'indicatorType': 'IP', 'value': '220.178.71.156', 'sourceKey': 'alienvault_otx'}],
                'responseMetadata': {'nextCursor': ''}}
    assert expected == page.to_dict(remove_nones=True)



import pytest
from requests.exceptions import HTTPError

from tests.conftest import BASE_URL

URL_ENDPOINT = BASE_URL + "/enclaves"


def test_trustar_get_enclaves(mocked_request, trustar, get_user_enclaves_fixture):
    mocked_request.get(url=f"{URL_ENDPOINT}", json=get_user_enclaves_fixture)
    assert len(trustar.get_user_enclaves()) == 4


def test_trustar_get_exception(mocked_request, trustar):
    mocked_request.get(url=f"{URL_ENDPOINT}", exc=HTTPError)
    with pytest.raises(HTTPError):
        trustar.get_user_enclaves()



# python 2 backwards compatibility
from __future__ import print_function
from builtins import object, str
from future import standard_library

# external imports
import json
import functools

# package imports
from .log import get_logger
from .models import CursorPage, PhishingIndicator, PhishingSubmission

# python 2 backwards compatibility
standard_library.install_aliases()

logger = get_logger(__name__)


class PhishingTriageClient(object):

    @staticmethod
    def remove_nones(d):
        """
        Removes None values from a dictionary.

        :param dict d: A dictionary
        """
        return {k: v for k, v in d.items() if v is not None}

    def get_phishing_submissions(self, from_time=None, to_time=None, priority_event_score=None,
                                 enclave_ids=None, status=None, cursor=None):
        """
        Fetches all phishing submissions that fit a given criteria.

        :param int from_time: Start of time window in milliseconds since epoch (optional)
        :param int to_time: End of time window in milliseconds since epoch (optional)
        :param list(int) priority_event_score: List of desired scores of phishing submission on a scale of 0-3
                                             (default: [3]).
        :param list(string) enclave_ids: List of enclave ids to pull submissions from.
                                         (defaults to all of a user's enclaves).
        :param list(string) status: List of statuses to filter submissions by. Options are 'UNRESOLVED', 'CONFIRMED',
                                    and 'IGNORED'. (default: ['UNRESOLVED']).
        :param string cursor: A Base64-encoded string that contains information on how to retrieve the next page.
                              If a cursor isn't passed, it will default to pageSize: 25, pageNumber: 0
        :return: CursorPage.generator - A generator object which can be used to paginate through |PhishingSubmission| data.
        """

        phishing_submissions_page_generator = self._get_phishing_submissions_page_generator(
            from_time=from_time,
            to_time=to_time,
            priority_event_score=priority_event_score,
            enclave_ids=enclave_ids,
            status=status,
            cursor=cursor
        )

        return CursorPage.get_generator(page_generator=phishing_submissions_page_generator)

    def _get_phishing_submissions_page_generator(self, from_time=None, to_time=None, priority_event_score=None,
                                                 enclave_ids=None, status=None, cursor=None):
        """
        Creates a generator from the |get_indicators_page| method that returns each successive page.

        :param int from_time: Start of time window in milliseconds since epoch (optional).
        :param int to_time: End of time window in milliseconds since epoch (optional).
        :param list(int) priority_event_score: List of desired scores of phishing submission on a scale of 0-3
                                                  (default: [3]).
        :param list(string) enclave_ids: A list of enclave IDs to filter by. (defaults to all of a user's enclaves)
        :param list(string) status: List of statuses to filter submissions by. Options are 'UNRESOLVED', 'CONFIRMED',
                                    and 'IGNORED'. (default: ['UNRESOLVED']).
        :param string cursor: A Base64-encoded string that contains information on how to retrieve the next page.
                              If a cursor isn't passed, it will default to pageSize: 25, pageNumber: 0
        """

        get_page = functools.partial(
            self.get_phishing_submissions_page,
            from_time=from_time,
            to_time=to_time,
            priority_event_score=priority_event_score,
            enclave_ids=enclave_ids,
            status=status,
        )

        return CursorPage.get_cursor_based_page_generator(get_page, cursor=cursor)

    def get_phishing_submissions_page(self, from_time=None, to_time=None, priority_event_score=None,
                                      enclave_ids=None, status=None, cursor=None, page_size=None):
        """
        Get a page of phishing submissions that match the given criteria.

        :param int from_time: Start of time window in milliseconds since epoch (optional).
        :param int to_time: End of time window in milliseconds since epoch (optional).
        :param list(int) priority_event_score: List of desired scores of phishing submission on a scale of 0-3
                                             (default: [3]).
        :param list(string) enclave_ids: A list of enclave IDs to filter by. (defaults to all of a user's enclaves)
        :param list(string) status: List of statuses to filter submissions by. Options are 'UNRESOLVED', 'CONFIRMED',
                                    and 'IGNORED'. (default: ['UNRESOLVED']).
        :param int page_size: Size of the page to be returned. Max value possible is 1000. Default is 25.
        :param string cursor: A Base64-encoded string that contains information on how to retrieve the next page.
                              If a cursor isn't passed, it will default to pageSize: 25, pageNumber: 0
        :return: |CursorPage| - An object representing a single page of |PhishingSubmission| objects.
        """
        params = {
            "pageSize" : page_size
        }

        data = self.remove_nones({
            'from': from_time,
            'to': to_time,
            'priorityEventScore': priority_event_score,
            'enclaveIds': enclave_ids,
            'status': status,
            'cursor': cursor
        })

        resp = self._client.post("triage/submissions", params=params, data=json.dumps(data))

        return CursorPage.from_dict(resp.json(), content_type=PhishingSubmission)

    def mark_triage_status(self, submission_id=None, status=None):
        """
        Marks a phishing email submission with one of the phishing namespace tags.

        :param string submission_id: ID of the email submission.
        :param string status: Triage status of submission.
        """

        if submission_id is None or not isinstance(submission_id, str):
            raise Exception("Please include ID of phishing email submission to mark triage status")

        params = {'status': status}

        return self._client.post("triage/submissions/{submission_id}/status"
                                 .format(submission_id=submission_id), params=params)

    def get_phishing_indicators(self, from_time=None, to_time=None, normalized_indicator_score=None,
                                priority_event_score=None, status=None, enclave_ids=None, cursor=None):
        """
        Get a page of phishing indicators that match the given criteria.

        :param int from_time: Start of time window in milliseconds since epoch (optional).
        :param int to_time: End of time window in milliseconds since epoch (optional).
        :param list(int) normalized_indicator_score: List of desired scores of intel sources on a scale of 0-3
                                                  (default: [3]).
        :param list(int) priority_event_score: List of desired scores of phishing indicators on a scale of 0-3
                                                  (default: [3]).
        :param list(string) enclave_ids: A list of enclave IDs to filter by. (defaults to all of a user's enclaves)
        :param list(string) status: List of statuses to filter indicators by. Options are 'UNRESOLVED', 'CONFIRMED',
                                    and 'IGNORED'. (default: ['UNRESOLVED']).
        :param string cursor: A Base64-encoded string that contains information on how to retrieve the next page.
                              If a cursor isn't passed, it will default to pageSize: 25, pageNumber: 0
        :return: CursorPage.generator - A generator object which can be used to paginate through |PhishingIndicator| data.
        """

        phishing_indicators_page_generator = self._get_phishing_indicators_page_generator(
            from_time=from_time,
            to_time=to_time,
            normalized_indicator_score=normalized_indicator_score,
            priority_event_score=priority_event_score,
            enclave_ids=enclave_ids,
            status=status,
            cursor=cursor
        )

        return CursorPage.get_generator(page_generator=phishing_indicators_page_generator)

    def _get_phishing_indicators_page_generator(self, from_time=None, to_time=None, normalized_indicator_score=None,
                                                priority_event_score=None, enclave_ids=None, status=None,
                                                cursor=None):
        """
        Creates a generator from the |get_indicators_page| method that returns each successive page.

        :param int from_time: Start of time window in milliseconds since epoch (optional).
        :param int to_time: End of time window in milliseconds since epoch (optional).
        :param list(int) normalized_indicator_score: List of desired scores of intel sources on a scale of 0-3
                                                  (default: [3]).
        :param list(int) priority_event_score: List of desired scores of phishing indicators on a scale of 0-3
                                                  (default: [3]).
        :param list(string) enclave_ids: A list of enclave IDs to filter by. (defaults to all of a user's enclaves)
        :param list(string) status: List of statuses to filter indicators by. Options are 'UNRESOLVED', 'CONFIRMED',
                                    and 'IGNORED'. (default: ['UNRESOLVED']).
        :param string cursor: A Base64-encoded string that contains information on how to retrieve the next page.
                              If a cursor isn't passed, it will default to pageSize: 25, pageNumber: 0
        """

        get_page = functools.partial(
            self.get_phishing_indicators_page,
            from_time=from_time,
            to_time=to_time,
            normalized_indicator_score=normalized_indicator_score,
            priority_event_score=priority_event_score,
            enclave_ids=enclave_ids,
            status=status,
        )

        return CursorPage.get_cursor_based_page_generator(get_page, cursor=cursor)

    def get_phishing_indicators_page(self, from_time=None, to_time=None, normalized_indicator_score=None,
                                     priority_event_score=None, enclave_ids=None, status=None, cursor=None,
                                     page_size=None):
        """
        Get a page of phishing indicators that match the given criteria.

        :param int from_time: Start of time window in milliseconds since epoch (optional).
        :param int to_time: End of time window in milliseconds since epoch (optional).
        :param list(int) normalized_indicator_score: List of desired scores of intel sources on a scale of 0-3
                                                  (default: [3]).
        :param list(int) priority_event_score: List of desired scores of phishing indicators on a scale of 0-3
                                             (default: [3]).
        :param list(string) enclave_ids: A list of enclave IDs to filter by.
        :param list(string) status: List of statuses to filter indicators by. Options are 'UNRESOLVED', 'CONFIRMED',
                                    and 'IGNORED'. (default: ['UNRESOLVED']).
        :param int page_size: Size of the page to be returned. Max value possible is 1000. Default is 25.
        :param string cursor: A Base64-encoded string that contains information on how to retrieve the next page.
                              If a cursor isn't passed, it will default to pageSize: 25, pageNumber: 0
        :return: |CursorPage| - An object representing a single page of |PhishingIndicator| objects.
        """
        params = {
            "pageSize" : page_size
        }

        data = self.remove_nones({
            'from': from_time,
            'to': to_time,
            'normalizedIndicatorScore': normalized_indicator_score,
            'priorityEventScore': priority_event_score,
            'enclaveIds': enclave_ids,
            'status': status,
            'cursor': cursor
        })

        resp = self._client.post("triage/indicators", params=params, data=json.dumps(data))

        return CursorPage.from_dict(resp.json(), content_type=PhishingIndicator)

    def from_dict(self):
        return {"submissionId": int(self)}



import time

import pytest
import requests_mock

from trustar import TruStar

BASE_URL = "/api/1.3"


@pytest.fixture
def mocked_request():
    with requests_mock.Mocker() as m:
        m.post(url="/oauth/token", text='{"access_token": "XXXXXXXXXXXXXXXXXXX"}')
        yield m


@pytest.fixture
def trustar():
    return TruStar(config_role='staging')


@pytest.fixture
def numbered_page():
    return {'items': [{"id": 1, "name": "mock"}],
            'pageNumber': 1, 'pageSize': 1,
            'totalElements': 1, 'hasNext': False}


@pytest.fixture
def current_time_millis():
    return int(time.time()) * 1000


@pytest.fixture
def milliseconds_in_a_day():
    return 24 * 60 * 60 * 1000


@pytest.fixture
def get_user_enclaves_fixture():
    return [{'id': 'xxxxxxx-xxx-xxxx-xxxx-xxxxxxxxxxx1', 'name': 'Community',
             'type': 'COMMUNITY', 'read': True, 'create': True, 'update': False},
            {'id': 'xxxxxxxx-xxxx-xxxx-xxxxxxxxxxxxxxx2', 'name': 'ncfta_stash',
             'type': 'INTERNAL', 'read': True, 'create': True, 'update': True},
            {'id': 'xxxxxxxx-xxxx-xxxx-xxxxxxxxxxxxxxx3',
             'name': 'not a real enclave',
             'type': 'INTERNAL', 'read': True, 'create': True, 'update': True},
            {'id': 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx4',
             'name': 'Nemo Research',
             'type': 'RESEARCH', 'read': True, 'create': False, 'update': False}]



# python 2 backwards compatibility
from __future__ import print_function

# package imports
from .base import ModelBase
from .page import Page

# external imports
import math


class CursorPage(Page):
    """
    This class models a page of items that would be found in the body of a response from an endpoint that uses
    pagination. Unlike the Page class, it uses cursor-based pagination instead of number-based pagination.

    :ivar items: The list of items of the page; i.e. a list of indicators, reports, etc.
    :ivar cursor: A Base64-encoded string that contains information on how to retrieve the next page.
                  If a cursor isn't passed, it will default to pageSize: 25, pageNumber: 0
    :ivar total_elements: The total number of elements on the server, e.g. the total number of elements across all
                          pages.  Note that it is possible for this value to change between pages,
                          since data can change between queries.
    """

    def __init__(self, items=None, response_metadata=None):
        """
        Instantiates an instance of the |CursorPage| class.
        """
        super(CursorPage, self).__init__(items=items)
        self.response_metadata = response_metadata

    @staticmethod
    def from_dict(page, content_type=None):
        """
        Create a |CursorPage| object from a dictionary.  This method is intended for internal use, to construct a
        |CursorPage| object from the body of a response json from a paginated endpoint.

        :param page: The dictionary.
        :param content_type: The class that the contents should be deserialized into.
        :return: The resulting |Page| object.
        """

        result = CursorPage(items=page.get('items'),
                            response_metadata=page.get('responseMetadata'))

        if content_type is not None:
            if not issubclass(content_type, ModelBase):
                raise ValueError("'content_type' must be a subclass of ModelBase.")

            result.items = [content_type.from_dict(item) for item in result.items]

        return result

    def to_dict(self, remove_nones=False):
        """
        Creates a dictionary representation of the page.

        :param remove_nones: Whether ``None`` values should be filtered out of the dictionary.  Defaults to ``False``.
        :return: A dictionary representation of the page.
        """
        return {
            'items': [n.to_dict(remove_nones) for n in self.items],
            'responseMetadata': self.response_metadata
        }

    @staticmethod
    def get_cursor_based_page_generator(get_page, cursor=None):
        """
        A page generator that uses cursor-based pagination.

        :param get_page: a function to get the next page, given values for from_time and to_time
        :param cursor: A Base64-encoded string that contains information on how to retrieve the next page.
                       If a cursor isn't passed, it will default to pageSize: 25, pageNumber: 0
        :return: a generator that yields each successive page
        """

        def get_next_cursor(page=None):
            """
            Retrieves the Base-64 encoded cursor string from a page.
            """

            next_cursor = str()
            response_metadata = str()
            page_dict = page.to_dict()
            if page_dict:
                response_metadata = page_dict.get('responseMetadata')
            if response_metadata:
                next_cursor = response_metadata.get('nextCursor')
            return next_cursor

        finished = False
        while not finished:
            # If cursor is None, no cursor value will be sent with request
            page = get_page(cursor=cursor)
            cursor = get_next_cursor(page)
            # If there are no more pages, cursor == "", therefore -> not "" == True
            finished = not cursor
            yield page



"""
Reads reports from a CSV and submits them to TruSTAR.
"""

from trustar import log, Report, TruStar
import csv

logger = log.get_logger(__name__)

# mapping of CSV column names to report fields
MAPPING = {
    "title": "name",
    "body": "content",
    "external_id": "id"
}

CSV_PATH = "reports.csv"

# initialize SDK
ts = TruStar()

# read in CSV
with open(CSV_PATH, 'r') as f:
    reader = csv.DictReader(f)

    # iterate over rows
    for row in reader:

        # define method to get report field from CSV row
        def get_field(field):
            return row.get(MAPPING.get(field))

        # construct report from CSV row
        report = Report(title=get_field('title'),
                        body=get_field('body'),
                        external_id=get_field('external_id'),
                        is_enclave=True,
                        enclave_ids=ts.enclave_ids)

        # submit report
        ts.submit_report(report)

        logger.info("Submitted report: %s" % report)



from tests.conftest import BASE_URL


def test_ping(mocked_request, trustar):
    # TODO tests different return values from the ping endpoint
    mocked_request.get(url=f"{BASE_URL}/ping")
    trustar.ping()





# python 2 backwards compatibility
from __future__ import print_function

# package imports
from .base import ModelBase


class Page(ModelBase):
    """
    This is the base class used to define default methods and attributes for all classes which will be used for
    pagination of the TruSTAR public API.

    :ivar items: The list of items of the page; i.e. a list of indicators, reports, etc.
    """

    def __init__(self, items=None):
        self.items = items

    def __len__(self):
        return len(self.items)

    def __iter__(self):
        return self.items.__iter__()

    def __getitem__(self, item):
        return self.items[item]

    @classmethod
    def get_generator(cls, page_generator):
        """
        Gets a generator for retrieving all results from a paginated endpoint.  Pass exactly one of ``page_generator``
        or ``func``.  This method is intended for internal use.

        :param page_generator: A generator to be used to generate each successive |Page|.
        :return: A generator that generates each successive element.
        """

        # yield each item in the page one by one;
        # once it is out, generate the next page
        for page in page_generator:
            for item in page.items:
                yield item



# python 2 backwards compatibility
from __future__ import print_function
from builtins import object, super
from future import standard_library
from six import string_types

# package imports
from .base import ModelBase


class IntelligenceSource(ModelBase):
    """
    Models an |IntelligenceSource_resource|.

    :ivar str key: A string that uniquely identifies the source, e.g. virustotal
    :ivar str name: A human-readable name of the source, as a human-readable string, e.g. "VirusTotal"
    """

    def __init__(self,
                 key=None,
                 name=None):

        self.key = key
        self.name = name

    @classmethod
    def from_dict(cls, source):
        """
        Create an |IntelligenceSource| object from a dictionary.

        :param source: The dictionary.
        :return: The |IntelligenceSource| object.
        """

        return IntelligenceSource(key=source.get('key'),
                                  name=source.get('name'))

    def to_dict(self, remove_nones=False):
        """
        Creates a dictionary representation of the source.

        :param remove_nones: Whether ``None`` values should be filtered out of the dictionary.  Defaults to ``False``.
        :return: A dictionary representation of the source.
        """

        return {
            'key': self.key,
            'name': self.name
        }



# python 2 backwards compatibility
from __future__ import print_function
from builtins import object, str
from future import standard_library
from six import string_types

# external imports
import json
from datetime import datetime
import functools

# package imports
from .log import get_logger
from .models import NumberedPage, Report, RedactedReport, DistributionType, IdType
from .utils import get_time_based_page_generator, DAY

# python 2 backwards compatibility
standard_library.install_aliases()

logger = get_logger(__name__)


class ReportClient(object):

    def get_report_details(self, report_id, id_type=None):
        """
        Retrieves a report by its ID.  Internal and external IDs are both allowed.

        :param str report_id: The ID of the incident report.
        :param str id_type: Indicates whether ID is internal or external.

        :return: The retrieved |Report| object.

        Example:

        >>> report = ts.get_report_details("1a09f14b-ef8c-443f-b082-9643071c522a")
        >>> print(report)
        {
          "id": "1a09f14b-ef8c-443f-b082-9643071c522a",
          "created": 1515571633505,
          "updated": 1515620420062,
          "reportBody": "Employee reported suspect email.  We had multiple reports of suspicious email overnight ...",
          "title": "Phishing Incident",
          "enclaveIds": [
            "ac6a0d17-7350-4410-bc57-9699521db992"
          ],
          "distributionType": "ENCLAVE",
          "timeBegan": 1479941278000
        }

        """

        params = {'idType': id_type}
        resp = self._client.get("reports/%s" % report_id, params=params)
        return Report.from_dict(resp.json())

    def get_reports_page(self, is_enclave=None, enclave_ids=None, tag=None, excluded_tags=None,
                         from_time=None, to_time=None):
        """
        Retrieves a page of reports, filtering by time window, distribution type, enclave association, and tag.
        The results are sorted by updated time.
        This method does not take ``page_number`` and ``page_size`` parameters.  Instead, each successive page must be
        found by adjusting the ``from_time`` and ``to_time`` parameters.

        Note:  This endpoint will only return reports from a time window of maximum size of 2 weeks. If you give a
        time window larger than 2 weeks, it will pull reports starting at 2 weeks before the "to" date, through the
        "to" date.

        :param boolean is_enclave: restrict reports to specific distribution type (optional - by default all accessible
            reports are returned).
        :param list(str) enclave_ids: list of enclave ids used to restrict reports to specific enclaves (optional - by
            default reports from all of user's enclaves are returned)
        :param list(str) tag: Name (or list of names) of tag(s) to filter reports by.  Only reports containing
            ALL of these tags will be returned.
        :param list(str) excluded_tags: Reports containing ANY of these tags will be excluded from the results.
        :param int from_time: start of time window in milliseconds since epoch (optional)
        :param int to_time: end of time window in milliseconds since epoch (optional)

        :return: A |NumberedPage| of |Report| objects.

        """

        distribution_type = None

        # explicitly compare to True and False to distinguish from None (which is treated as False in a conditional)
        if is_enclave:
            distribution_type = DistributionType.ENCLAVE
        elif not is_enclave:
            distribution_type = DistributionType.COMMUNITY

        if enclave_ids is None:
            enclave_ids = self.enclave_ids

        params = {
            'from': from_time,
            'to': to_time,
            'distributionType': distribution_type,
            'enclaveIds': enclave_ids,
            'tags': tag,
            'excludedTags': excluded_tags
        }
        resp = self._client.get("reports", params=params)
        result = NumberedPage.from_dict(resp.json(), content_type=Report)

        # create a NumberedPage object from the dict
        return result

    def submit_report(self, report):
        """
        Submit a report.

        * If ``report.is_enclave`` is ``True``, then the report will be submitted to the enclaves
          identified by ``report.enclaves``; if that field is ``None``, then the enclave IDs registered with this
          |TruStar| object will be used.
        * If ``report.time_began`` is ``None``, then the current time will be used.

        :param report: The |Report| object that was submitted, with the ``id`` field updated based
            on values from the response.

        Example:

        >>> report = Report(title="Suspicious Activity",
        >>>                 body="We have been receiving suspicious requests from 169.178.68.63.",
        >>>                 enclave_ids=["602d4795-31cd-44f9-a85d-f33cb869145a"])
        >>> report = ts.submit_report(report)
        >>> print(report.id)
        ac6a0d17-7350-4410-bc57-9699521db992
        >>> print(report.title)
        Suspicious Activity
        """

        # make distribution type default to "enclave"
        if report.is_enclave is None:
            report.is_enclave = True

        if report.enclave_ids is None:
            # use configured enclave_ids by default if distribution type is ENCLAVE
            if report.is_enclave:
                report.enclave_ids = self.enclave_ids
            # if distribution type is COMMUNITY, API still expects non-null list of enclaves
            else:
                report.enclave_ids = []

        if report.is_enclave and len(report.enclave_ids) == 0:
            raise Exception("Cannot submit a report of distribution type 'ENCLAVE' with an empty set of enclaves.")

        # default time began is current time
        if report.time_began is None:
            report.set_time_began(datetime.now())

        data = json.dumps(report.to_dict())
        resp = self._client.post("reports", data=data, timeout=60)

        # get report id from response body
        report_id = resp.content

        if isinstance(report_id, bytes):
            report_id = report_id.decode('utf-8')

        report.id = report_id

        return report

    def update_report(self, report):
        """
        Updates the report identified by the ``report.id`` field; if this field does not exist, then
        ``report.external_id`` will be used if it exists.  Any other fields on ``report`` that are not ``None``
        will overwrite values on the report in TruSTAR's system.   Any fields that are  ``None`` will simply be ignored;
        their values will be unchanged.

        :param report: A |Report| object with the updated values.
        :return: The |Report| object.

        Example:

        >>> report = ts.get_report_details(report_id)
        >>> print(report.title)
        Old Title
        >>> report.title = "Changed title"
        >>> updated_report = ts.update_report(report)
        >>> print(updated_report.title)
        Changed Title
        """

        # default to interal ID type if ID field is present
        if report.id is not None:
            id_type = IdType.INTERNAL
            report_id = report.id
        # if no ID field is present, but external ID field is, default to external ID type
        elif report.external_id is not None:
            id_type = IdType.EXTERNAL
            report_id = report.external_id
        # if no ID fields exist, raise exception
        else:
            raise Exception("Cannot update report without either an ID or an external ID.")

        # not allowed to update value of 'reportId', so remove it
        report_dict = {k: v for k, v in report.to_dict().items() if k != 'reportId'}

        params = {'idType': id_type}

        data = json.dumps(report.to_dict())
        self._client.put("reports/%s" % report_id, data=data, params=params)

        return report

    def delete_report(self, report_id, id_type=None):
        """
        Deletes the report with the given ID.

        :param report_id: the ID of the report to delete
        :param id_type: indicates whether the ID is internal or an external ID provided by the user
        :return: the response object

        Example:

        >>> response = ts.delete_report("4d1fcaee-5009-4620-b239-2b22c3992b80")
        """

        params = {'idType': id_type}
        self._client.delete("reports/%s" % report_id, params=params)

    def copy_report(self, src_report_id, dest_enclave_id, from_provided_submission=False, report=None, tags=None):
        """
        Copy a report to another enclave.  All properties of the report, including tags, will be copied.
        A reference to the original report will still be stored on the child, allowing the system to track the
        relationship between the original report and copies made from it.

        If the ``from_provided_submission`` parameter is ``True``, then edits can be applied to the copied report.  This
        is useful in cases where the body or title must be redacted first, or the list of tags needs to be altered for
        the copy.  In this case, a |Report| object and a list of tag names must be provided, which will fill out the
        copied report.  A reference to the original report will still be stored on the copy.
        **NOTE:** Partial edits are not allowed.  ALL fields must be filled out on this object, and the
        fields from the original report will completely ignored.

        :param str src_report_id: the ID of the report to copy
        :param str dest_enclave_id: the ID of the enclave to copy the report to
        :param boolean from_provided_submission: whether to apply edits from a supplied report object and list of tags
        :param Report report: (required if ``from_provided_submission`` is ``True``) a report object containing an edited version to use as the copy.
            This allows information to be redacted, or other arbitrary edits to be made to the copied version.
            **NOTE:** Partial edits are not allowed.  ALL fields must be filled out on this object, and the fields from
            the original report will completely ignored.
        :param list(str) tags: (required if ``from_provided_submission`` is ``True``) a list of tags to use if ``from_provided_submission`` is ``True``.
            **NOTE:** if ``from_provided_submission`` is True, the tags from the source report will be completely
            ignored, and this list of tags will be used instead.  MUST be provided if ``from_provided_submission`` is ``True``.
        :return: the ID of the newly-created copy
        """

        params = {
            'destEnclaveId': dest_enclave_id,
            'copyFromProvidedSubmission': from_provided_submission
        }

        # determine if edits are being made to the copy
        if from_provided_submission:
            # ensure an edited version of the report has been provided
            if not report:
                raise Exception("Cannot copy from provided submission without providing a report object")
            # ensure an edited list of tags has been provided
            if not tags:
                raise Exception("Cannot copy from provided submission without providing a list of tags")

            # form the JSON dictionary of the report
            body = report.to_dict()
            # add the list of tags to the JSON
            # NOTE: this field on the report object cannot be used in other endpoints on this API version
            body['tags'] = tags
        else:
            body = None

        response = self._client.post('reports/copy/{id}'.format(id=src_report_id), params=params, data=json.dumps(body))
        return response.json().get('id')

    def move_report(self, report_id, dest_enclave_id):
        """
        Move a report from one enclave to another.

        **NOTE:** All tags will be moved, as well.

        :param report_id: the ID of the report to move
        :param dest_enclave_id: the ID of the enclave to move the report to
        :return: the ID of the report
        """

        params = {
            'destEnclaveId': dest_enclave_id
        }

        response = self._client.post('reports/move/{id}'.format(id=report_id), params=params)
        return response.json().get('id')

    def get_correlated_report_ids(self, indicators):
        """
        DEPRECATED!
        Retrieves a list of the IDs of all TruSTAR reports that contain the searched indicators.

        :param indicators: A list of indicator values to retrieve correlated reports for.
        :return: The list of IDs of reports that correlated.

        Example:

        >>> report_ids = ts.get_correlated_report_ids(["wannacry", "www.evil.com"])
        >>> print(report_ids)
        ["e3bc6921-e2c8-42eb-829e-eea8da2d3f36", "4d04804f-ff82-4a0b-8586-c42aef2f6f73"]
        """

        params = {'indicators': indicators}
        resp = self._client.get("reports/correlate", params=params)
        return resp.json()

    def get_correlated_reports_page(self, indicators, enclave_ids=None, is_enclave=True,
                                    page_size=None, page_number=None):
        """
        Retrieves a page of all TruSTAR reports that contain the searched indicators.

        :param indicators: A list of indicator values to retrieve correlated reports for.
        :param enclave_ids: The enclaves to search in.
        :param is_enclave: Whether to search enclave reports or community reports.
        :param int page_number: the page number to get.
        :param int page_size: the size of the page to be returned.
        :return: The list of IDs of reports that correlated.

        Example:

        >>> reports = ts.get_correlated_reports_page(["wannacry", "www.evil.com"]).items
        >>> print([report.id for report in reports])
        ["e3bc6921-e2c8-42eb-829e-eea8da2d3f36", "4d04804f-ff82-4a0b-8586-c42aef2f6f73"]
        """

        if is_enclave:
            distribution_type = DistributionType.ENCLAVE
        else:
            distribution_type = DistributionType.COMMUNITY

        params = {
            'indicators': indicators,
            'enclaveIds': enclave_ids,
            'distributionType': distribution_type,
            'pageNumber': page_number,
            'pageSize': page_size
        }
        resp = self._client.get("reports/correlated", params=params)

        return NumberedPage.from_dict(resp.json(), content_type=Report)

    def search_reports_page(self, search_term=None,
                            enclave_ids=None,
                            from_time=None,
                            to_time=None,
                            tags=None,
                            excluded_tags=None,
                            page_size=None,
                            page_number=None):
        """
        Search for reports containing a search term.

        :param str search_term: The term to search for.  If empty, no search term will be applied.  Otherwise, must
            be at least 3 characters.
        :param list(str) enclave_ids: list of enclave ids used to restrict reports to specific enclaves (optional - by
            default reports from all of user's enclaves are returned)
        :param int from_time: start of time window in milliseconds since epoch (optional)
        :param int to_time: end of time window in milliseconds since epoch (optional)
        :param list(str) tags: Name (or list of names) of tag(s) to filter reports by.  Only reports containing
            ALL of these tags will be returned. (optional)
        :param list(str) excluded_tags: Reports containing ANY of these tags will be excluded from the results.
        :param int page_number: the page number to get. (optional)
        :param int page_size: the size of the page to be returned.
        :return: a |NumberedPage| of |Report| objects.  *NOTE*:  The bodies of these reports will be ``None``.
        """

        body = {
            'searchTerm': search_term
        }

        params = {
            'enclaveIds': enclave_ids,
            'from': from_time,
            'to': to_time,
            'tags': tags,
            'excludedTags': excluded_tags,
            'pageSize': page_size,
            'pageNumber': page_number
        }

        resp = self._client.post("reports/search", params=params, data=json.dumps(body))
        page = NumberedPage.from_dict(resp.json(), content_type=Report)

        return page

    def _get_reports_page_generator(self, is_enclave=None, enclave_ids=None, tag=None, excluded_tags=None,
                                    from_time=None, to_time=None):
        """
        Creates a generator from the |get_reports_page| method that returns each successive page.

        :param boolean is_enclave: restrict reports to specific distribution type (optional - by default all accessible
            reports are returned).
        :param list(str) enclave_ids: list of enclave ids used to restrict reports to specific
            enclaves (optional - by default reports from all enclaves are returned)
        :param str tag: name of tag to filter reports by.  if a tag with this name exists in more than one enclave
            indicated in ``enclave_ids``, the request will fail.  handle this by making separate requests for each
            enclave ID if necessary.
        :param int from_time: start of time window in milliseconds since epoch
        :param int to_time: end of time window in milliseconds since epoch (optional, defaults to current time)
        :return: The generator.
        """

        def get_next_to_time(result, to_time):
            """
            For each page, get the timestamp of the earliest report in the result set.  The next query will use this
            timestamp as the end of its interval.  This endpoint limits queries to 1 day.  If the result set is
            empty, subtract 1 day from the to_time for the next interval.

            :param result: the result set of the previous call
            :param to_time: the to_time of the previous call
            :return: the next to_time
            """

            if len(result.items) > 0:
                return result.items[-1].updated - 1
            else:
                return to_time - DAY

        get_page = functools.partial(self.get_reports_page, is_enclave, enclave_ids, tag, excluded_tags)
        return get_time_based_page_generator(
            get_page=get_page,
            get_next_to_time=get_next_to_time,
            from_time=from_time,
            to_time=to_time
        )

    def get_reports(self, is_enclave=None, enclave_ids=None, tag=None, excluded_tags=None, from_time=None, to_time=None):
        """
        Uses the |get_reports_page| method to create a generator that returns each successive report as a trustar
        report object.

        :param boolean is_enclave: restrict reports to specific distribution type (optional - by default all accessible
            reports are returned).
        :param list(str) enclave_ids: list of enclave ids used to restrict reports to specific
            enclaves (optional - by default reports from all enclaves are returned)
        :param list(str) tag: a list of tags; only reports containing ALL of these tags will be returned. 
            If a tag with this name exists in more than one enclave in the list passed as the ``enclave_ids``
            argument, the request will fail.  Handle this by making separate requests for each
            enclave ID if necessary.
        :param list(str) excluded_tags: a list of tags; reports containing ANY of these tags will not be returned. 
        :param int from_time: start of time window in milliseconds since epoch (optional)
        :param int to_time: end of time window in milliseconds since epoch (optional)
        :return: A generator of Report objects.

        Note:  If a report contains all of the tags in the list passed as argument to the 'tag' parameter and also 
        contains any (1 or more) of the tags in the list passed as argument to the 'excluded_tags' parameter, that 
        report will not be returned by this function.  
        
        Example:

        >>> page = ts.get_reports(is_enclave=True, tag="malicious", from_time=1425695711000, to_time=1514185311000)
        >>> for report in reports: print(report.id)
        '661583cb-a6a7-4cbd-8a90-01578fa4da89'
        'da131660-2708-4c8a-926e-f91fb5dbbc62'
        '2e3400d6-fa37-4a8c-bc2f-155aaa02ae5a'
        '38064828-d3db-4fff-8ab8-e0e3b304ff44'
        'dbf26104-cee5-4ca4-bdbf-a01d0178c007'

        """

        return NumberedPage.get_generator(page_generator=self._get_reports_page_generator(is_enclave, enclave_ids, tag,
                                                                                  excluded_tags, from_time, to_time))

    def _get_correlated_reports_page_generator(self, indicators, enclave_ids=None, is_enclave=True,
                                               start_page=0, page_size=None):
        """
        Creates a generator from the |get_correlated_reports_page| method that returns each
        successive page.

        :param indicators: A list of indicator values to retrieve correlated reports for.
        :param enclave_ids:
        :param is_enclave:
        :return: The generator.
        """

        get_page = functools.partial(self.get_correlated_reports_page, indicators, enclave_ids, is_enclave)
        return NumberedPage.get_page_generator(get_page, start_page, page_size)

    def get_correlated_reports(self, indicators, enclave_ids=None, is_enclave=True):
        """
        Uses the |get_correlated_reports_page| method to create a generator that returns each successive report.

        :param indicators: A list of indicator values to retrieve correlated reports for.
        :param enclave_ids: The enclaves to search in.
        :param is_enclave: Whether to search enclave reports or community reports.
        :return: The generator.
        """

        return NumberedPage.get_generator(page_generator=self._get_correlated_reports_page_generator(indicators,
                                                                                             enclave_ids,
                                                                                             is_enclave))

    def _search_reports_page_generator(self, search_term=None,
                                       enclave_ids=None,
                                       from_time=None,
                                       to_time=None,
                                       tags=None,
                                       excluded_tags=None,
                                       start_page=0,
                                       page_size=None):
        """
        Creates a generator from the |search_reports_page| method that returns each successive page.

        :param str search_term: The term to search for.  If empty, no search term will be applied.  Otherwise, must
            be at least 3 characters.
        :param list(str) enclave_ids: list of enclave ids used to restrict reports to specific enclaves (optional - by
            default reports from all of user's enclaves are returned)
        :param int from_time: start of time window in milliseconds since epoch (optional)
        :param int to_time: end of time window in milliseconds since epoch (optional)
        :param list(str) tags: Name (or list of names) of tag(s) to filter reports by.  Only reports containing
            ALL of these tags will be returned. (optional)
        :param list(str) excluded_tags: Reports containing ANY of these tags will be excluded from the results.
        :param int start_page: The page to start on.
        :param page_size: The size of each page.
        :return: The generator.
        """

        get_page = functools.partial(self.search_reports_page, search_term, enclave_ids, from_time, to_time, tags,
                                     excluded_tags)
        return NumberedPage.get_page_generator(get_page, start_page, page_size)

    def search_reports(self, search_term=None,
                       enclave_ids=None,
                       from_time=None,
                       to_time=None,
                       tags=None,
                       excluded_tags=None):
        """
        Uses the |search_reports_page| method to create a generator that returns each successive report.

        :param str search_term: The term to search for.  If empty, no search term will be applied.  Otherwise, must
            be at least 3 characters.
        :param list(str) enclave_ids: list of enclave ids used to restrict reports to specific enclaves (optional - by
            default reports from all of user's enclaves are returned)
        :param int from_time: start of time window in milliseconds since epoch (optional)
        :param int to_time: end of time window in milliseconds since epoch (optional)
        :param list(str) tags: Name (or list of names) of tag(s) to filter reports by.  Only reports containing
            ALL of these tags will be returned. (optional)
        :param list(str) excluded_tags: Reports containing ANY of these tags will be excluded from the results.
        :return: The generator of Report objects.  Note that the body attributes of these reports will be ``None``.
        """

        return NumberedPage.get_generator(page_generator=self._search_reports_page_generator(search_term, enclave_ids,
                                                                                     from_time, to_time, tags,
                                                                                     excluded_tags))

    def redact_report(self, title=None, report_body=None):
        """
        Redacts a report's title and body.

        :param str title: The title of the report to apply redaction to.
        :param str report_body: The body of the report to apply redaction to.
        :return: a |RedactedReport| object.
        """

        body = {
            'title': title,
            'reportBody': report_body
        }

        resp = self._client.post("redaction/report", data=json.dumps(body))

        return RedactedReport.from_dict(resp.json())
    
    def get_report_deeplink(self, report):
        """
        Retrieves the Station's report deeplink.

        :param report: A |Report| or a str object.
        :return: A report URL object.

        Example:

        >>> report = "fcda196b-eb30-4b59-83b8-a25ab6d70d17"
        >>> deeplink = ts.get_report_deeplink(report)
        >>> isinstance(report, str) or isinstance(report, Report)
        True
        >>> isinstance(deeplink, str)
        True
        >>> print(deeplink)
        """

        # default to interal ID if report ID field is present
        # else treat report as an ID string
        try:
            report_id = report.id
        except AttributeError:
            report_id = report
        deeplink = "{}/constellation/reports/{}".format(self._client.station, report_id)

        return deeplink



# python 2 backwards compatibility
from __future__ import print_function
from builtins import super

# package imports
from .base import ModelBase


class RedactedReport(ModelBase):
    """
    Models the response of the `POST /redaction/report` endpoint.

    :ivar title: the report title
    :ivar body: the report body
    """

    def __init__(self,
                 title=None,
                 body=None):
        """
        Constructs a RedactedReport object.

        :param title: the report title
        :param body: the report body
        """

        self.title = title
        self.body = body

    def to_dict(self, remove_nones=False):
        """
        Creates a dictionary representation of the object.

        :param remove_nones: Whether ``None`` values should be filtered out of the dictionary.  Defaults to ``False``.
        :return: A dictionary representation of the redacted report.
        """

        if remove_nones:
            redacted_report_dict = super(RedactedReport, self).to_dict(remove_nones=True)
        else:
            redacted_report_dict = {
                'title': self.title,
                'reportBody': self.body
            }

        return redacted_report_dict

    @classmethod
    def from_dict(cls, redacted_report):
        """
        Create a RedactedReport object from a dictionary.  This method is intended for internal use, to construct a
        :class:`RedactedReport` object from the body of a response json.  It expects the keys of the dictionary to match those
        of the json that would be found in a response to an API call such as ``POST redaction/report``.

        :param redacted_report: The dictionary.
        :return: The RedactionReport object.
        """

        return RedactedReport(title=redacted_report.get('title'),
                              body=redacted_report.get('reportBody'))



import pytest

from trustar import Report, IdType
from tests.conftest import BASE_URL

FAKE_REPORT_ID = 45
URL_ENDPOINT = BASE_URL + "/reports"
ORIGINAL_TAG = 'original_tag'


@pytest.fixture
def report(trustar, current_time_millis, milliseconds_in_a_day):
    original_report = Report(title="Report 1",
                             id=FAKE_REPORT_ID,
                             body="Blah blah blah",
                             time_began=current_time_millis - milliseconds_in_a_day,
                             enclave_ids=trustar.enclave_ids)
    return original_report


@pytest.fixture
def original_report(mocked_request, trustar, report):
    # TODO test different report types, and different return types
    mocked_request.post(url=URL_ENDPOINT, text=str(report.id))
    o_r = trustar.submit_report(report=report)
    return o_r


@pytest.fixture()
def copied_report(trustar, original_report):
    copied_report_id = FAKE_REPORT_ID + 1
    dest_enclave_id = trustar.enclave_ids[0]
    return {"id": copied_report_id,
            "reportBody": original_report.body,
            "title": original_report.title,
            "enclaveIds": dest_enclave_id}


def test_delete_report(mocked_request, trustar):
    mocked_request.delete(url=f"{URL_ENDPOINT}/{FAKE_REPORT_ID}")
    trustar.delete_report(FAKE_REPORT_ID)


def test_add_enclave_tag(mocked_request, trustar, original_report):
    mocked_request.post(url=f"{URL_ENDPOINT}/{original_report.id}/alter-tags", json={"id": original_report.id})
    name = trustar.add_enclave_tag(original_report.id, ORIGINAL_TAG, None)
    assert name == ORIGINAL_TAG


def test_copy_report(mocked_request, trustar, original_report):
    dest_enclave_id = trustar.enclave_ids[0]
    _url = f"{URL_ENDPOINT}/copy/{FAKE_REPORT_ID}?destEnclaveId={dest_enclave_id}&copyFromProvidedSubmission=False"
    mocked_request.post(url=_url, json={"id": FAKE_REPORT_ID + 1})
    # TODO test exceptions from_provided_submission True
    copied_report_id = trustar.copy_report(src_report_id=original_report.id,
                                           dest_enclave_id=dest_enclave_id,
                                           from_provided_submission=False)
    assert copied_report_id == FAKE_REPORT_ID + 1


def test_get_enclave_tags(mocked_request, trustar):
    copied_report_id = FAKE_REPORT_ID + 1
    mocked_request.get(url=f"{URL_ENDPOINT}/{copied_report_id}/tags", json=[{"name": ORIGINAL_TAG}])
    original_tags = trustar.get_enclave_tags(copied_report_id)
    assert {ORIGINAL_TAG} == set(t.name for t in original_tags)


def test_get_report_details(mocked_request, trustar, original_report, copied_report):
    mocked_request.get(url=f"{URL_ENDPOINT}/{copied_report['id']}", json=copied_report)
    copied_report_result = trustar.get_report_details(copied_report['id'])
    # TODO: add a __eq__ method to the report class
    assert original_report.title == copied_report_result.title
    assert original_report.body == copied_report_result.body
    assert copied_report_result.enclave_ids[0] == trustar.enclave_ids[0]


def test_move_report(mocked_request, trustar, copied_report):
    _url = f"{URL_ENDPOINT}/move/{copied_report['id']}?destEnclaveId={trustar.enclave_ids[0]}"
    mocked_request.post(url=_url, json=copied_report)
    moved_report_id = trustar.move_report(report_id=copied_report['id'], dest_enclave_id=trustar.enclave_ids[0])
    assert moved_report_id == moved_report_id


def test_delete_enclave_tag(mocked_request, trustar, original_report):
    tag_id = id_type = "XXX-XXXX"
    mocked_request.post(url=f"{URL_ENDPOINT}/{FAKE_REPORT_ID}/alter-tags", json={"id": original_report.id})
    result = trustar.delete_enclave_tag(report_id=original_report.id, tag_id=tag_id, id_type=id_type)
    assert result == original_report.id


def test_get_report_deeplink(mocked_request, trustar, original_report):
    mocked_request.delete(url=f"{URL_ENDPOINT}/{FAKE_REPORT_ID}")
    # TODO check for exceptions
    response = trustar.get_report_deeplink(report=original_report.id)
    assert response == f"{trustar._client.station}/constellation/reports/{original_report.id}"


def test_update_report(mocked_request, trustar, original_report):
    original_report.body = "Bleh bleh bleh"
    mocked_request.put(url=f"{URL_ENDPOINT}/{FAKE_REPORT_ID}?idType={IdType.INTERNAL}")
    # TODO check for exceptions
    report = trustar.update_report(report=original_report)
    # TODO __eq__ method for reports
    assert original_report.body == report.body


def test_search_reports(mocked_request, trustar, numbered_page):
    mocked_request.post(url=f"{URL_ENDPOINT}/search?pageNumber=0", json=numbered_page)
    reports = trustar.search_reports("abc")
    assert len(list(reports)) > 0


def test_get_correlated_reports(mocked_request, trustar):
    indicators = ("evil", "wannacry")
    _url = f"{URL_ENDPOINT}/correlate?indicators={indicators[0]}&indicators={indicators[1]}"
    expected = {"response": "fake"}
    mocked_request.get(url=_url, json=expected)
    result = trustar.get_correlated_report_ids(indicators)
    assert expected == result


def test_get_reports(mocked_request, trustar, numbered_page, current_time_millis):
    from_time = current_time_millis - 60
    tag = "XXXXXX"
    numbered_page["items"][0]["updated"] = from_time
    _url = f"{URL_ENDPOINT}?from={from_time}&to={current_time_millis}&" \
           f"distributionType=COMMUNITY&enclaveIds={trustar.enclave_ids[0]}&tags={tag}"
    mocked_request.get(url=_url, json=numbered_page)
    result = trustar.get_reports(tag=tag, from_time=from_time, to_time=current_time_millis)
    # TODO check the content
    assert len(list(result)) > 0



import pytest

from tests.conftest import BASE_URL
from trustar import IndicatorType, NumberedPage, Indicator, Tag

URL_ENDPOINT = BASE_URL + "/indicators"
URL_ENDPOINT_WHITELIST = BASE_URL + "/whitelist"


@pytest.fixture
def indicators(current_time_millis, milliseconds_in_a_day):
    return [Indicator(value="1.5.8.7",
                      first_seen=current_time_millis - milliseconds_in_a_day,
                      last_seen=current_time_millis,
                      sightings=100, source="Somewhere", notes="This is a note."),
            Indicator(value="1.5.8.9",
                      first_seen=current_time_millis - milliseconds_in_a_day,
                      last_seen=current_time_millis,
                      sightings=50, source="Somewhere else", notes="This is another note.")]


@pytest.fixture
def indicators_dict(indicators):
    return [i.to_dict() for i in indicators]


@pytest.fixture
def tags(trustar):
    return [Tag(name="tag_1", enclave_id=trustar.enclave_ids[0]), Tag(name="tag_2", enclave_id=trustar.enclave_ids[0])]


def test_get_indicator_details(mocked_request, trustar, indicators_dict):
    _url = f"{URL_ENDPOINT}/details?indicatorValues={indicators_dict[0]['value']}" \
           f"&indicatorValues={indicators_dict[1]['value']}"
    mocked_request.get(url=_url, json=indicators_dict)
    details = trustar.get_indicator_details([indicators_dict[0]['value'], indicators_dict[1]['value']])
    assert details[0].to_dict() == indicators_dict[0]


@pytest.mark.parametrize("indicator_type", (IndicatorType.CVE, IndicatorType.MALWARE, None))
def test_community_trends(mocked_request, trustar, indicators_dict, indicator_type):
    _url = f"{URL_ENDPOINT}/community-trending"
    if indicator_type:
        _url += f"?type={indicator_type}"
    indicators_dict[0]["indicatorType"] = indicator_type
    mocked_request.get(url=_url, json=indicators_dict)
    result = trustar.get_community_trends(indicator_type=indicator_type)
    assert result[0].type == indicator_type


def test_add_to_whitelist(mocked_request, trustar, indicators):
    expected = [i.to_dict(remove_nones=True) for i in indicators]
    mocked_request.post(url=f"{URL_ENDPOINT_WHITELIST}", json=expected)
    result = trustar.add_terms_to_whitelist(expected)
    # TODO implement __eq__ method for indicators
    assert [i.to_dict(remove_nones=True) for i in result] == expected


def test_get_whitelist_page(mocked_request, trustar, numbered_page, indicators_dict):
    numbered_page["items"] = indicators_dict
    mocked_request.get(url=f"{URL_ENDPOINT_WHITELIST}", json=numbered_page)
    result = trustar.get_whitelist_page()
    assert result.to_dict() == numbered_page


def test_delete_indicator_from_whitelist(mocked_request, trustar, indicators):
    _url = f"{URL_ENDPOINT_WHITELIST}?value={indicators[0].value}" \
           f"&firstSeen={indicators[0].first_seen}&lastSeen={indicators[0].last_seen}" \
           f"&source={indicators[0].source}&notes={indicators[0].notes}"
    mocked_request.delete(url=_url)
    trustar.delete_indicator_from_whitelist(indicator=indicators[0])


def test_get_related_indicators(mocked_request, trustar, numbered_page):
    _url = f"{URL_ENDPOINT}/related?indicators=evil&indicators=1.2.3.4&indicators=wannacry"
    mocked_request.get(url=_url, json=numbered_page)
    result = trustar.get_related_indicators_page(indicators=("evil", "1.2.3.4", "wannacry"))
    assert isinstance(result, NumberedPage)


def test_search_indicators(mocked_request, trustar, numbered_page):
    mocked_request.post(f"{URL_ENDPOINT}/search?pageNumber=0", json=numbered_page)
    indicators = trustar.search_indicators("abc")
    assert len(list(indicators)) > 0


def test_submit_indicators(mocked_request, trustar, indicators, tags):
    mocked_request.post(f"{URL_ENDPOINT}")
    # TODO test exceptions
    trustar.submit_indicators(indicators=indicators, tags=tags)


def test_get_indicator_metadata(mocked_request, trustar, indicators, tags):
    indicators[0].tags = [tags[0]]
    mocked_request.post(f"{URL_ENDPOINT}/metadata", json=[indicators[0].to_dict(remove_nones=True)])
    expected = trustar.get_indicator_metadata(value=indicators[0].value)
    # TODO implement __eq__ for indicator and tags.
    assert expected['indicator'].to_dict() == indicators[0].to_dict()
    assert expected['tags'][0].to_dict() == indicators[0].tags[0].to_dict()
    assert expected['enclaveIds'] == indicators[0].enclave_ids


@pytest.mark.parametrize("indicator_type", (IndicatorType.IP, IndicatorType.URL, None))
def test_get_indicators_metadata(mocked_request, trustar, indicators, indicators_dict, indicator_type):
    for i in indicators:
        i.type = indicator_type
    mocked_request.post(url=f"{URL_ENDPOINT}/metadata", json=indicators_dict)
    metadata = trustar.get_indicators_metadata(indicators)
    assert len(metadata) == 2


def test_add_tag_to_indicator(mocked_request, trustar, tags):
    expected = tags[0].to_dict()
    mocked_request.post(f"{URL_ENDPOINT}/tags", json=expected)
    result = trustar.add_indicator_tag("blah.com", name="indicator_tag", enclave_id=trustar.enclave_ids[0])
    assert result.to_dict() == expected


def test_get_indicator_tags(mocked_request, trustar, tags):
    expected = [tags[0].to_dict()]
    _url = f"{URL_ENDPOINT}/tags?enclaveIds={trustar.enclave_ids[0]}"
    mocked_request.get(url=_url, json=expected)
    response = trustar.get_all_indicator_tags()
    assert expected[0] == response[0].to_dict()


def test_delete_tag_from_indicator(mocked_request, trustar):
    metadata = "xxxxx"
    tag_id = 12345
    mocked_request.delete(url=f"{URL_ENDPOINT}/tags/{tag_id}?value={metadata}")
    trustar.delete_indicator_tag(metadata, tag_id=tag_id)
